﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;


namespace RobvanderWoude
{
	internal class HasMouse
	{
		static readonly string progver = "2.00";


		static int Main( string[] args )
		{
			bool quiet = false;


			#region Parse Command Line

			if ( args.Length == 1 )
			{
				if ( args[0].ToUpper( ) == "/Q" )
				{
					quiet = true;
				}
				else
				{
					return ShowHelp( );
				}
			}
			if ( args.Length > 1 )
			{
				return ShowHelp( );
			}

			#endregion Parse Command Line


			// Count total number of mouse buttons
			int buttons = int.Parse( typeof( SystemInformation ).GetProperty( "MouseButtons" ).GetValue( null, null ).ToString( ) );
			if ( buttons == 0 )
			{
				// 0 buttons means no mouse
				if ( !quiet )
				{
					Console.Write( "SystemInformation MouseButtons returned " );
					Console.ForegroundColor = ConsoleColor.Red;
					Console.Write( "0" );
					Console.ResetColor( );
					Console.Write( " => " );
					Console.ForegroundColor = ConsoleColor.Red;
					Console.Write( "no" );
					Console.ResetColor( );
					Console.WriteLine( " mouse" );
				}
				return 0;
			}
			else
			{
				// 1 or more buttons means at least 1 mouse
				if ( !quiet )
				{
					Console.Write( "SystemInformation MouseButtons returned " );
					Console.ForegroundColor = ConsoleColor.Green;
					Console.Write( buttons );
					Console.ResetColor( );
					Console.Write( " => at least " );
					Console.ForegroundColor = ConsoleColor.Green;
					Console.Write( "1" );
					Console.ResetColor( );
					Console.WriteLine( " mouse" );
				}
				return 1;
			}
		}


		public static int ShowHelp( params string[] errmsg )
		{
			#region Error Message

			if ( errmsg.Length > 0 )
			{
				List<string> errargs = new List<string>( errmsg );
				errargs.RemoveAt( 0 );
				Console.Error.WriteLine( );
				Console.ForegroundColor = ConsoleColor.Red;
				Console.Error.Write( "ERROR:\t" );
				Console.ForegroundColor = ConsoleColor.White;
				Console.Error.WriteLine( errmsg[0], errargs.ToArray( ) );
				Console.ResetColor( );
			}

			#endregion Error Message


			#region Help Text

			/*
			HasMouse.exe,  Version 2.00
			Check if a mouse is available at this very moment by counting mouse buttons

			Usage:   HasMouse.exe  [ /Q ]

			Where:   /Q            Quiet mode (no screen output)

			Notes:   As of version 2.00 this program no longer uses P/Invoke to count
			         mouse buttons.
			         The program's return code equals 1 if at least 1 mouse is detected,
			         0 in case of no mice, or -1 in case of (command line) errors.
			
			Written by Rob van der Woude
			https://www.robvanderwoude.com
			*/

			#endregion Help Text


			#region Display Help Text

			Console.Error.WriteLine( );

			Console.Error.WriteLine( "HasMouse.exe,  Version {0}", progver );

			Console.Error.WriteLine( "Check if a mouse is available at this very moment by counting mouse buttons" );

			Console.Error.WriteLine( );

			Console.Error.Write( "Usage:   " );
			Console.ForegroundColor = ConsoleColor.White;
			Console.Error.WriteLine( "HasMouse.exe  [ /Q ]" );
			Console.ResetColor( );

			Console.Error.WriteLine( );

			Console.Error.Write( "Where:   " );
			Console.ForegroundColor = ConsoleColor.White;
			Console.Error.Write( "/Q            Q" );
			Console.ResetColor( );
			Console.Error.WriteLine( "uiet mode (no screen output)" );

			Console.Error.WriteLine( );

			Console.Error.WriteLine( "Notes:   As of version 2.00 this program no longer uses P/Invoke to count" );

			Console.Error.WriteLine( "         mouse buttons." );

			Console.Error.WriteLine( "         The program's return code equals 1 if at least 1 mouse is detected," );

			Console.Error.WriteLine( "         0 in case of no mice, or -1 in case of (command line) errors." );

			Console.Error.WriteLine( );

			Console.Error.WriteLine( "Written by Rob van der Woude" );

			Console.Error.WriteLine( "https://www.robvanderwoude.com" );

			#endregion Display Help Text


			return -1;
		}
	}
}
